LOGGER_CONFIG = {
    'log_dir': '/Users/sinri/code/nehushtan/log',
}
IMAP_CONFIG = {

}

MYSQL_CONFIG = {

}

SMTP_CONFIG = {

}

ALIYUN_SLS_CONFIG = {
    'project': 'zechariah',
    'logstore': 'zechariah-test',
    # 'endpoint': 'cn-hangzhou-intranet.log.aliyuncs.com',
    'endpoint': 'cn-hangzhou.log.aliyuncs.com',
    'source': '[IP]',
    'accessKeyId': 'XXX',
    'accessKeySecret': 'YYY',
}
