class NehushtanHTTPConstant:
    """
    Since 0.4.0
    """

    METHOD_HEAD = "HEAD"
    METHOD_GET = "GET"
    METHOD_POST = "POST"
    METHOD_PUT = "PUT"
    METHOD_DELETE = "DELETE"
    METHOD_OPTIONS = "OPTIONS"
    METHOD_PATCH = "PATCH"

    HEADER_CONTENT_TYPE = 'Content-Type'

    HEADER_CONTENT_TYPE_VALUE_TEXT = 'text/plain'
    HEADER_CONTENT_TYPE_VALUE_HTML = 'text/html'
    HEADER_CONTENT_TYPE_VALUE_JSON = 'application/json'

    REPLY_CODE_OK = 'OK'
    REPLY_CODE_FAIL = 'FAIL'
